/*

	SPECTRUM PRO CHROME DEMO by Kas Thomas 
	Copyright 2016 ID TECH. All rights reserved. 

*/

var MY_HID_VENDOR_ID  = 0x0acd; // VID
var MY_HID_PRODUCT_ID = 0x4010; // Spectrum Pro

var POLL_INTERVAL = 20; // 20 msec

var DEVICE_INFO = 	{"vendorId": MY_HID_VENDOR_ID, "productId": MY_HID_PRODUCT_ID };

var connectionId = 	null;

var ktNode = 		null;  

// This code binds the HTML UI to the program logic:
document.addEventListener('DOMContentLoaded', function () {
    document.getElementById('actionButton').addEventListener( 'click', myAction );
    document.getElementById('clearButton').addEventListener( 'click', clearAll );
    document.getElementById('selection').addEventListener( 'change', writeToUserText );
    document.getElementById('connectButton').addEventListener( 'click', connect );
    
});

function myAction() {

    var cmd = document.getElementById("cmd");
    var userText = cmd.value;

	// check if user text looks like 02xxxxxx
    if ( userText.indexOf("02") == 0  ) {

	writeOutput("\r\n" + "Sent: " + userText);

	try {
		var buffer = commandStringToArrayBuffer( userText );
		chrome.hid.send( connectionId, 0, buffer, function(){  } );
	}
	catch(e) {
		writeOutput( "Action failed: " + e.toString() );
	}

    } 
    else
	try {
		var cmd = formatCommand( userText ); 
		writeOutput("\r\nSent: " + cmd);
		var buffer = commandStringToArrayBuffer( cmd );
		chrome.hid.send( connectionId, 0, buffer, function(){} );
	}
	catch(e) {
		writeOutput( "Action failed: " + e.toString() );
	}
	
}

function clearAll() {
    document.getElementById("cmd").value = "";
    document.getElementById("kt").value = "";
    document.querySelector( "#output" ).innerHTML = '';	
}

function getSelection() {

    var selection = document.getElementById("selection").value;
    return selection;
}

function writeToUserText( str ) {

	document.getElementById("cmd").value = getSelection();
}

function getOutputNode() {

	if (document)
		ktNode = document.getElementById("kt");
        else
		console.log("document was null.");

}

// write to the app's HTML page
function writeOutput( value ) {

	if (!ktNode)
		getOutputNode();

	if (ktNode) {
		ktNode.value += value ;
		ktNode.scrollTop = ktNode.scrollHeight; // scroll to bottom
	}
	else
		console.log("ktNode was null.");
}

// return LRC as hex string;
// input should look like ["1A","22","3C", etc.]
function LRC(ar) {
   var lrc=0;
   for (var i=0;i<ar.length;i++)
     lrc ^= Number("0x" + ar[i]); 
   var hex = lrc.toString(16).toUpperCase();
   return (hex.length < 2) ? "0" + hex : hex;
} 

// return 8-bit checksum as hex string
function checksum(ar) {
   var sum=0;
   for (var i=0;i<ar.length;i++)
     sum += Number("0x" + ar[i]); 
   var hex = (255 & sum).toString(16).toUpperCase();
   return (hex.length < 2) ? "0" + hex : hex;
}

function decimalToHexString( n ) {
   var hex = n.toString(16).toUpperCase();
   return ( hex.length < 2) ? "0" + hex : hex;
}

// Assumes cmd looks like "78 46 01" 
// Here, we need to package it as: 
//    STX + length bytes + cmd + lrc + sum + ETX
function formatCommand( cmd ) {

	var STX = "02"; var ETX = "03";

	// first, collapse spaces
	var input = cmd.replace(/\s+/g,"");

	// break "784601" into [78,46,01]
	var inputArray = input.match(/../g);

	var lrc = LRC( inputArray );
	var sum = checksum( inputArray );
	
	var len = decimalToHexString( inputArray.length ) + "00";

	return STX + len + inputArray.join("") + lrc + sum + ETX;
}

// convert ArrayBuffer to String
function arrayBufferToString( array ) {
    return String.fromCharCode.apply(null, new Uint8Array( array ) );
}

// convert ArrayBuffer to hex string
function arrayBufferToHex( array ) {

	var output = "";
	var dv = new DataView( array );
	var len = dv.byteLength;
	for ( var i = 0; i < len; i++) {

		var item = dv.getUint8( i );
		var hex = item.toString( 16 );
		if (hex.length < 2) 
			hex = '0' + hex;
		output += hex + " ";
	}
	return output;
}

// convert USB report to hex string
function usbReportToHexString( usb ) {

	var usbArray = usb.split(" ");
	var length = usbArray.shift();
	length = 1 * ("0x" + length);
	usbArray = usbArray.slice( 0, length );
	return usbArray.join("");
}

// Take s = "020400724609013cc203" and turn it
// into [2, 4, 0, 114, 70, 9, 1, 60, 194, 3]
function hexStringToNumericArray( s ) {

	var array = s.match(/../g);
	var output = [];

	for ( var i = 0; i < array.length; i++ )
		output.push( 1 * ("0x"+array[i] ) );

	return output;
}

// Convert "020400724609013cc203" (hex string) into an ArrayBuffer prepended with length
function commandStringToArrayBuffer( s ) {

	var numericArray = hexStringToNumericArray(s);

	// make the first byte the length:
	numericArray.unshift( numericArray.length );

	var uint8 = new Uint8Array( numericArray);
	return uint8.buffer;
}


var WAIT_TIME = 500;

// Accumulate multi-report data here
responseData = { "busy":false, 

		 "data":"" };



function report() { 
	var data = responseData.data.match(/../g);
	if ( !data )
		return;
	var text = data.join(" ");
	if ( responseData.data.length > 63 ) {
		console.log( "\nMulti-report payload:\n" + text );
		writeOutput( "\n\nMulti-report payload:\n" + text + "\n");
		
	}

	responseData.busy = false;

	responseData.data = "";

}



function collect( text ) {

	if ( responseData.busy == false )

		setTimeout( report, WAIT_TIME );

	responseData.busy = true;

	responseData.data += text;

}


// this will be 2nd arg to chrome.hid.receive
var myReceiver = function( reportID, data ) {

	DATA = data; // DEBUG

	value = "";

        if (data != null) {

            // Convert Byte into Ascii to follow the format of our device
            value = arrayBufferToString(data);

	    // DEBUG: Write to console window
	    if ( value != "" && value ) 
               console.log('ReportID: ' + reportID + '\r\n     Data: ' + value);
	           
	    // Write to HTML window
	    var output = "\r\n";
	    output += "Received: ";
	    var text = usbReportToHexString( arrayBufferToHex( data ) );
	    output += text;
	    collect( text ); // accumulate multi-report responses here, call report() on it later
 	    output += "\r\n";
	    output += value;
	    writeOutput( output + "\r\n" );
            writeOutput( arrayBufferToHex( data ) + "\r\n" );

	    showPollResults(); // check if 76 46 25 was responded to
	   
        }

        setTimeout( myDevicePoll, POLL_INTERVAL );
};

// Enable auto-detect. 
// If user unplugs, then replugs, the device
// it should auto-detect & connect
chrome.hid.onDeviceAdded.addListener( 
   function( deviceInfo ) {	
	MY_HID_VENDOR_ID = DEVICE_INFO.vendorId = deviceInfo.vendorId;
	MY_HID_PRODUCT_ID = DEVICE_INFO.productId = deviceInfo.productId;
	writeOutput( deviceInfo.productName + " detected...\n" );
	connect();
   } 
);

// poll the device
// NOTE: A ref to this function is the arg to initializeHid(), below
var myDevicePoll = function() {

    chrome.hid.receive( connectionId, myReceiver );
}


// Must pass a polling function 'pollHid' to this function
function initializeHid( pollHid ) {

    // Try to open the USB HID device
    chrome.hid.getDevices( DEVICE_INFO, function(devices) {
        if (!devices || !devices.length) {
   	    console.log('device not found; try hitting the Reload link on chrome://extensions page');
	    writeOutput('device not found; try hitting the Reload link on chrome://extensions page' +
			'\n-- or simply unplug/replug the device into the computer\'s USB port.\n');
   	    return;
   	}
   	console.log('Found device: ' + devices[0].deviceId);
   	myHidDevice = devices[0].deviceId;

        // Connect to the HID device
   	chrome.hid.connect(myHidDevice, function(connection) {

            console.log('Connected to the HID device!');
	    writeOutput('Connected to the HID device!');

   	    connectionId = connection.connectionId;

            // Poll the USB HID Interrupt pipe
            pollHid();
   	});
    });
}

initializeHid( myDevicePoll );  // This executes on code-load

function connect() {
   initializeHid( myDevicePoll );
}

// ============================= HANDLE POLL CARD READER COMMAND =============

function showPollResults( ) {

	var output = document.querySelector( "#output" );

	var wasPolled = responseData.data.match(/06674625/g);
	if ( !wasPolled ) {
		output.innerHTML = "";
		return null;
	}
	
	var data = responseData.data.match( /../g);
	if ( data ) {

		data = data.slice( 9, 15 );
		var bytes = [];
		for (var k=0; k < data.length;k++)
			bytes.push( Number( "0x" + data[k] ) );
		output.innerHTML = parsePollResults( bytes );
	}
	return null;
}

function parsePollResults( bytes ) {

var legendByte0 = ["Key is initialized if set to 1 (MFK and RSA Key pairs)",
"DATA_DUKPT_Key valid if set to 1",
"HOST_CR_RKL_DUKPT_Key valid if set to 1",
"Device Manufacturing CA data valid  if set to 1",
"HOST_CR_MAC Keys valid (Authenticated) if 1",
"HOST_CR_MASTER_DUKPT_Key valid if set to 1",
"Authenticated with Host if set to 1",
"Device Manufacturing Secure data valid if set to 1"];

var legendByte1 = ["HSM_DUKPT_KEY valid if set to 1",
"CR_PINPAD_RKL_DUKPT_KEY valid if set to 1",
"PIN Pairing DUKPT Key valid if set to to 1",
"DATA Pairing DUKPT Key valid if set to 1",
"CR_PINPAD_MAC Keys valid (Authenticated) if 1",
"CR_PINPAD_MASTER_DUKPT_KEY valid if set to 1",
"Authenticated with Pinpad if set to 1",
"Firmware Key valid  if set to 1"
];

var legendByte2 = ["Removal Tamper Error if set to 1",
"Tamper Switch#2 Error if set to 1",
"Maxq Firmware Auth Error if set to 1",
"Voltage Sensor Error if set to 1",
"Temperature Error if set to 1",
"Battery Backup Error if set to 1",
"Tamper Switch#1 Error if set to 1",
"K21  Firmware Auth Error if set to 1"
];


// byte 3 is battery status

var legendByte4 = ["Tamper detector active if set to 1",
"Removal sensor active  if set to 1",
"Latch Mechanism Active if set to 1",
"Card Present (Seated Switch) if set to 1",
"Removal sensor disconnected if set to 1",
"Card Insert (Front Switch) if set to 1",
"Mag Data Present if set to 1",
"Log is full if set to 1"

];

var legendByte5 = [
"0-Maxq Manufacture FW; 1-Maxq Production",
"0-K21 Manufacture FW; 1-K21 Production FW",
"MSR Header connected if set to 1",
"PINPAD connected if set to 1",
"Contactless  available if set to 1",
"Host connected if set to 1",
"Chip card reader available if set to 1",
"SAM available  if set to 1"
];



	// inner method
	function showBitmapLegend( byte, legend ) {

   		var html = [];
   		var bits = "1-------";
   		var flag = 128;
   		for (var i = 0; i < 8; i++, flag>>=1) {
    		   bits = bits.replace( /[01]/,( flag & byte ) ? "1" : "0");
                                if ( flag & byte ) html.push("<mark>");
   		   html.push( bits + "  " + legend[i] + "<br/>");
                                if ( flag & byte ) html.push("</mark>");
     		   bits = "-" + bits.substring(0,7);
   		}
   		return  '<pre style="font-size:8pt;">' +  html.join("") + '</pre>' ;
	}


var html = "<h3>Poll Reader Results</h3>";

html += "Byte 0:<br/>";
html += showBitmapLegend( bytes[0], legendByte0 );


html += "Byte 1:<br/>";
html += showBitmapLegend( bytes[1], legendByte1 );

html += "Byte 2:<br/>";
html += showBitmapLegend( bytes[2], legendByte2 );

html += "(Byte 3 is battery voltage)<br/>";
html += bytes[3].toString(16).substring(0,1) + "." + bytes[3].toString(16).substring(1) + "<br/><br/>";

html += "Byte 4:<br/>";
html += showBitmapLegend( bytes[4], legendByte4 );

html += "Byte 5:<br/>";
html += showBitmapLegend( bytes[5], legendByte5 );

return html;

}

